/* Zaurus keyboard simulation kernel module.
   This kernel module waits for blocks of 2
   bytes to be sent to /dev/simkbd, the first
   byte being a keyboard scancode and the
   second one 0 or 1 if the simulated key is
   up or down, then injects the scancode into
   the kernel as if it came from the real
   keyboard.

   The main purpose of this program is to be
   used in combination with fbvncserver, so the
   Zaurus can be remote-controlled from a VNC
   client.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version
   2 of the License, or (at your option) any later version.

   (c) Pierre-Philippe Coupard, 30/01/2002 */
#define MODULE
#include <linux/module.h>
#include <linux/fs.h>

#define KBDSIM_MAJOR 61
#define KBDSIM_MINOR 0



/* Licensing */
#ifdef MODULE_LICENSE
MODULE_LICENSE("GPL");
#endif



/* Prototypes */
ssize_t kbdsim_write(struct file *file,const char *buf,
			size_t count,loff_t *offp);
int kbdsim_open(struct inode *inode,struct file *filp);
int kbdsim_release(struct inode *inode,struct file *filp);
extern void handle_scancode(unsigned char scancode,int down);



/* Global vars */
struct file_operations kbdsim_fops =
{
	owner:THIS_MODULE,
	write:kbdsim_write,
	open:kbdsim_open,
	release:kbdsim_release,
};
int nb_opens;			/* Nb of processes opening this driver */



/* Open the device file */
int kbdsim_open(struct inode *inode,struct file *filp)
{
  int minor=MINOR(inode->i_rdev);

  /* Is it our minor ? */
  if(minor!=KBDSIM_MINOR)
    return(-ENODEV);

  /* We don't allow more than one process to open the device file */
  if(nb_opens)
    return(-EBUSY);

  nb_opens++;
  MOD_INC_USE_COUNT;

  return(0);
}



/* Close the device file */
int kbdsim_release(struct inode *inode,struct file *filp)
{
  int minor=MINOR(inode->i_rdev);

  /* Is it our minor ? */
  if(minor!=KBDSIM_MINOR)
    return(-ENODEV);

  nb_opens--;
  MOD_DEC_USE_COUNT;

  return(0);
}



/* Recover data written to the char device */
ssize_t kbdsim_write(struct file *file,const char *buf,
			size_t count,loff_t *offp)
{
  int down;

  /* We only accept single bytes and ignore anything else */
  if(count==2)
  {
    down=buf[1]?1:0;
    handle_scancode(buf[0],down);
  }

  return(count);
}



/* Init modules */
int init_module(void)
{
  return(register_chrdev(KBDSIM_MAJOR,"kbdsim",&kbdsim_fops));
}



/* Cleanup modules */
void cleanup_module(void)
{
  unregister_chrdev(KBDSIM_MAJOR,"kbdsim");
}
