/* Zaurus framebuffer VNC server

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version
   2 of the License, or (at your option) any later version.

   (c) Pierre-Philippe Coupard, 27/01/2002
*/
#ifndef IPAQ
#include "leds.h"



/* Uncomment the next line if you don't want to do the framebuffer mirroring
   in fast (but non-portable) assembly */
/*#define PORTABLE_RFB_MIRRORING 1*/



#define LCM_BASE	0x40000000
#define _LCM( x )	((x) + LCM_BASE)

#define LCM_p2v( x )    ((x) - LCM_BASE + 0xf0000000)

#define _LCM_LPT0	_LCM( 0xe8 )	/* LEDPWM0 timer */
#define _LCM_LPT1	_LCM( 0xec )	/* LEDPWM1 timer */

#define LCM_LPT0	(*((volatile unsigned short int *) LCM_p2v (_LCM_LPT0)))
#define LCM_LPT1	(*((volatile unsigned short int *) LCM_p2v (_LCM_LPT1)))



/* Variables */
static int blinkctr[2];



#ifdef PORTABLE_RFB_MIRRORING
#define DELAY_MULTIPLIER 2
#else
#define DELAY_MULTIPLIER 5
#endif



/* Return the LED state. We emulate the LoCoMo LED blinking here as well, so we
   only return the led values. 0x1 is for LED1, 0x2 is for LED2, and we return
   LED1 | LED2 */
int get_led_states()
{
  int lcm_vals[2];
  int ledvals[2];
  int i;

  /* Get the current LCM led controller values */
  lcm_vals[0]=LCM_LPT0;
  lcm_vals[1]=LCM_LPT1;

  /* Work out the current LED values from the current blinking stage */
  for(i=0;i<2;i++)
  {
    if(blinkctr[i] < (((lcm_vals[i]&0x70))>>4) * DELAY_MULTIPLIER)
      ledvals[i]=1;
    else
      ledvals[i]=0;
    blinkctr[i]++;

    if(blinkctr[i] >= (((lcm_vals[i]&0x70)>>4)+(lcm_vals[i]&0x07)) *
						DELAY_MULTIPLIER)
      blinkctr[i]=0;
  }

  /* Override a blinking value if a LED is ON or OFF all the time */
  for(i=0;i<2;i++)
  {
    if(lcm_vals[i] & 0x80)	/* ON all the time ? */
      ledvals[i]=1;

    if(lcm_vals[i] & 0x08)	/* OFF all the time ? */
      ledvals[i]=0;
  }

  return((ledvals[0]?0x1:0) | (ledvals[1]?0x2:0));
}
#endif
