/* Zaurus framebuffer VNC server

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version
   2 of the License, or (at your option) any later version.

   (c) Pierre-Philippe Coupard, 02/02/2002
*/
#include <stdio.h>
#include <stdlib.h>
#include <jpeglib.h>
#include <jerror.h>
#include <setjmp.h>
#include "loadjpeg.h"



/* Structures */
struct jpeg_error
{
  struct jpeg_error_mgr pub;
  sigjmp_buf setjmp_buffer;
};



/* Functions */
void jpeg_error_exit(j_common_ptr cinfo)
{
  struct jpeg_error *errmgr;

  errmgr=(struct jpeg_error *) cinfo->err;
  cinfo->err->output_message(cinfo);
  siglongjmp(errmgr->setjmp_buffer, 1);

  return;
}



/* Load a jpeg file. Note : the output buffer is in 555 format (15bpp) */
unsigned short int *loadjpeg(FILE *infile,int *width,int *height)
{
  struct jpeg_decompress_struct cinfo;
  struct jpeg_error jerr;
  unsigned short int *img_15bpp;
  JSAMPARRAY buffer;
  int row_stride;
  int bpix;
  int i,x,y;

  /* Decompress the image */
  cinfo.err=jpeg_std_error(&jerr.pub);
  jerr.pub.error_exit=jpeg_error_exit;

  jpeg_create_decompress (&cinfo);
  jpeg_stdio_src(&cinfo,infile);
  jpeg_read_header(&cinfo,FALSE);
  cinfo.do_fancy_upsampling=FALSE;
  cinfo.do_block_smoothing=FALSE;
  jpeg_start_decompress(&cinfo);
  *width=cinfo.output_width;
  *height=cinfo.output_height;

  img_15bpp = malloc((*width)*(*height)*2);

  y=0;
  row_stride=cinfo.output_width*cinfo.output_components;
  buffer=(*cinfo.mem->alloc_sarray)((j_common_ptr)&cinfo,JPOOL_IMAGE,
					 row_stride,1);
  
  bpix=cinfo.output_components;
  	
  while(cinfo.output_scanline<cinfo.output_height)
  {
    jpeg_read_scanlines (&cinfo,buffer,1);
    x=0;
    for(i=0;i<bpix*cinfo.output_width;i+=bpix)
    {
      img_15bpp[y*(*width)+x]=((buffer[0][i+2]>>3)<<10)+
				((buffer[0][i+1]>>3)<<5)+(buffer[0][i]>>3);
      x++;
    }
    y++;
  }
  
  jpeg_finish_decompress (&cinfo);
  jpeg_destroy_decompress (&cinfo);

  return(img_15bpp);
}
